@@ -15,8 +15,8 @@ import uuid
 import six
 from six.moves.urllib import parse as urlparse
 
+from keystoneclient.tests.unit import client_fixtures
 from keystoneclient.tests.unit import utils
-from keystoneclient.v3 import client
 
 
 TestResponse = utils.TestResponse
@@ -48,6 +48,7 @@ class UnauthenticatedTestCase(utils.TestCase):
 class TestCase(UnauthenticatedTestCase):
 
     TEST_ADMIN_IDENTITY_ENDPOINT = "http://127.0.0.1:35357/v3"
+    TEST_PUBLIC_IDENTITY_ENDPOINT = "http://127.0.0.1:5000/v3"
 
     TEST_SERVICE_CATALOG = [{
         "endpoints": [{
@@ -97,7 +98,7 @@ class TestCase(UnauthenticatedTestCase):
         "name": "glance"
     }, {
         "endpoints": [{
-            "url": "http://127.0.0.1:5000/v3",
+            "url": TEST_PUBLIC_IDENTITY_ENDPOINT,
             "region": "RegionOne",
             "interface": "public"
         }, {
@@ -127,15 +128,8 @@ class TestCase(UnauthenticatedTestCase):
         "type": "object-store"
     }]
 
-    def setUp(self):
-        super(TestCase, self).setUp()
-        self.client = client.Client(username=self.TEST_USER,
-                                    token=self.TEST_TOKEN,
-                                    tenant_name=self.TEST_TENANT_NAME,
-                                    auth_url=self.TEST_URL,
-                                    endpoint=self.TEST_URL)
-
     def stub_auth(self, subject_token=None, **kwargs):
+
         if not subject_token:
             subject_token = self.TEST_TOKEN
 
@@ -152,6 +146,44 @@ class TestCase(UnauthenticatedTestCase):
         self.stub_url('POST', ['auth', 'tokens'], **kwargs)
 
 
+class ClientTestCase(utils.ClientTestCaseMixin, TestCase):
+
+    ORIGINAL_CLIENT_TYPE = 'original'
+    KSC_SESSION_CLIENT_TYPE = 'ksc-session'
+    KSA_SESSION_CLIENT_TYPE = 'ksa-session'
+
+    scenarios = [
+        (
+            ORIGINAL_CLIENT_TYPE, {
+                'client_fixture_class': client_fixtures.OriginalV3,
+                'client_type': ORIGINAL_CLIENT_TYPE
+            }
+        ),
+        (
+            KSC_SESSION_CLIENT_TYPE, {
+                'client_fixture_class': client_fixtures.KscSessionV3,
+                'client_type': KSC_SESSION_CLIENT_TYPE
+            }
+        ),
+        (
+            KSA_SESSION_CLIENT_TYPE, {
+                'client_fixture_class': client_fixtures.KsaSessionV3,
+                'client_type': KSA_SESSION_CLIENT_TYPE
+            }
+        )
+
+    ]
+
+    @property
+    def is_original_client(self):
+        return self.client_type == self.ORIGINAL_CLIENT_TYPE
+
+    @property
+    def is_session_client(self):
+        return self.client_type in (self.KSC_SESSION_CLIENT_TYPE,
+                                    self.KSA_SESSION_CLIENT_TYPE)
+
+
 class CrudTests(object):
     key = None
     collection_key = None
@@ -164,11 +196,16 @@ class CrudTests(object):
         kwargs.setdefault(uuid.uuid4().hex, uuid.uuid4().hex)
         return kwargs
 
-    def encode(self, entity):
+    def encode(self, entity, truncated=None):
+        encoded = {}
+        if truncated is not None:
+            encoded['truncated'] = truncated
         if isinstance(entity, dict):
-            return {self.key: entity}
+            encoded[self.key] = entity
+            return encoded
         if isinstance(entity, list):
-            return {self.collection_key: entity}
+            encoded[self.collection_key] = entity
+            return encoded
         raise NotImplementedError('Are you sure you want to encode that?')
 
     def stub_entity(self, method, parts=None, entity=None, id=None, **kwargs):
@@ -245,14 +282,36 @@ class CrudTests(object):
 
         return expected_path
 
-    def test_list(self, ref_list=None, expected_path=None,
-                  expected_query=None, **filter_kwargs):
+    def test_list_by_id(self, ref=None, **filter_kwargs):
+        """Test ``entities.list(id=x)`` being rewritten as ``GET /v3/entities/x``.
+
+        This tests an edge case of each manager's list() implementation, to
+        ensure that it "does the right thing" when users call ``.list()``
+        when they should have used ``.get()``.
+
+        """
+        if 'id' not in filter_kwargs:
+            ref = ref or self.new_ref()
+            filter_kwargs['id'] = ref['id']
+
+        self.assertRaises(TypeError, self.manager.list, **filter_kwargs)
+
+    def _test_list(self, ref_list=None, expected_path=None,
+                   expected_query=None, truncated=None, **filter_kwargs):
         ref_list = ref_list or [self.new_ref(), self.new_ref()]
         expected_path = self._get_expected_path(expected_path)
 
-        self.requests_mock.get(urlparse.urljoin(self.TEST_URL, expected_path),
-                               json=self.encode(ref_list))
+        # We want to catch all cases: when `truncated` is not returned by the
+        # server, when it's False and when it's True.
+        # Attribute `truncated` of the returned list-like object should exist
+        # in all these cases. It should be False if the server returned a list
+        # without the flag.
+        expected_truncated = False
+        if truncated:
+            expected_truncated = truncated
 
+        self.requests_mock.get(urlparse.urljoin(self.TEST_URL, expected_path),
+                               json=self.encode(ref_list, truncated=truncated))
         returned_list = self.manager.list(**filter_kwargs)
         self.assertEqual(len(ref_list), len(returned_list))
         [self.assertIsInstance(r, self.model) for r in returned_list]
@@ -271,6 +330,20 @@ class CrudTests(object):
         for key in qs_args:
             self.assertIn(key, qs_args_expected)
 
+        self.assertEqual(expected_truncated, returned_list.truncated)
+
+    def test_list(self, ref_list=None, expected_path=None,
+                  expected_query=None, **filter_kwargs):
+        # test simple list, without any truncation
+        self._test_list(ref_list, expected_path, expected_query,
+                        **filter_kwargs)
+        # test when a server returned a list with truncated=False
+        self._test_list(ref_list, expected_path, expected_query,
+                        truncated=False, **filter_kwargs)
+        # test when a server returned a list with truncated=True
+        self._test_list(ref_list, expected_path, expected_query,
+                        truncated=True, **filter_kwargs)
+
     def test_list_params(self):
         ref_list = [self.new_ref()]
         filter_kwargs = {uuid.uuid4().hex: uuid.uuid4().hex}
